#include "helper_funcs.h"
#include <alsa/asoundlib.h>
#include <arpa/inet.h>
#include <cerrno>
#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <fcntl.h>
#include <iostream>
#include <netdb.h>
#include <netinet/in.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/wait.h>
#include <unistd.h>

namespace alsa {
int get_volume(const char *selem_name) {
  long min, max, volume;
  snd_mixer_t *handle;
  snd_mixer_selem_id_t *sid;

  // Open mixer
  snd_mixer_open(&handle, 0);
  snd_mixer_attach(handle,
                   "default"); // Use "default" for the default sound card
  snd_mixer_selem_register(handle, NULL, NULL);
  snd_mixer_load(handle);

  // Allocate and set mixer element ID
  snd_mixer_selem_id_alloca(&sid);
  snd_mixer_selem_id_set_name(sid, selem_name);

  // Find the mixer element
  snd_mixer_elem_t *elem = snd_mixer_find_selem(handle, sid);

  int volume_percent;
  if (elem) {
    // Get the playback volume range
    snd_mixer_selem_get_playback_volume_range(elem, &min, &max);

    // Get the current playback volume for front left channel
    snd_mixer_selem_get_playback_volume(elem, SND_MIXER_SCHN_FRONT_LEFT,
                                        &volume);

    // Calculate the volume percentage
    volume_percent = (volume - min) * 100 / (max - min);

  } else {
    std::cerr << "Mixer element not found\n";
    snd_mixer_close(handle);
    safe_exit(ENODEV);
  }

  // Clean up
  snd_mixer_close(handle);
  return volume_percent;
}

void set_volume(const char *selem_name, long volume_percent) {
  long min, max;
  snd_mixer_t *handle;
  snd_mixer_selem_id_t *sid;

  // Open mixer
  snd_mixer_open(&handle, 0);
  snd_mixer_attach(handle, "default"); // Use "default" instead of hw:0
  snd_mixer_selem_register(handle, NULL, NULL);
  snd_mixer_load(handle);

  // Allocate and set mixer element ID
  snd_mixer_selem_id_alloca(&sid);
  snd_mixer_selem_id_set_name(sid, selem_name);

  // Find the mixer element
  snd_mixer_elem_t *elem = snd_mixer_find_selem(handle, sid);

  if (elem) {
    // Get the playback volume range
    snd_mixer_selem_get_playback_volume_range(elem, &min, &max);

    // Calculate the new volume value
    long volume = min + (volume_percent * (max - min) / 100);

    // Set the playback volume
    snd_mixer_selem_set_playback_volume_all(elem, volume);
    printf("Volume set to %ld%%\n", volume_percent);
  } else {
    fprintf(stderr, "Mixer element not found\n");
  }

  // Clean up
  snd_mixer_close(handle);
}

int main(int argc, char *argv[]) {
  if (argc != 3) {
    fprintf(stderr, "Usage: %s <selem_name> <volume_percent>\n", argv[0]);
    return 1;
  }

  const char *selem_name = argv[1];
  long volume_percent = atol(argv[2]);

  set_volume(selem_name, volume_percent);

  return 0;
}
} // namespace alsa